@@ -1,4 +1,4 @@
-# lfs.sh,v 1.7 2009/10/13 19:11:01 lacos Exp
+# lfs.sh,v 1.8 2010/02/18 22:20:30 lacos Exp
 
 # Shell script to determine c89 configuration strings for large file support.
 # Specify one of CFLAGS, LDFLAGS, LIBS in $1.
@@ -37,7 +37,7 @@ do
     if ! getconf --version 2>&1 | grep -E -q 'getconf \((GNU libc|EGLIBC)\)' \
         || ! getconf $SPEC"_$1" 2>/dev/null
     then
-      getconf -v _$SPEC $SPEC"_$1"
+      getconf -v $SPEC $SPEC"_$1"
     fi
     exit 0
   fi
