@@ -1,4 +1,4 @@
-/* main.c,v 1.69 2009/12/28 13:53:40 lacos Exp */
+/* main.c,v 1.75 2010/03/03 01:12:45 lacos Exp */
 
 #include <unistd.h>          /* unlink() */
 #include <sys/types.h>       /* kill() */
@@ -39,16 +39,33 @@ static pid_t pid;
 static void
 bailout(void)
 {
+  sigset_t tmp_set;
+
   if (pthread_equal(pthread_self(), main_thread)) {
     if (0 != opathn) {
       (void)unlink(opathn);
     }
-  }
-  else {
-    if (0 == kill(pid, SIGUSR1)) {
-      pthread_exit(0);
+
+    if (0 == sigemptyset(&tmp_set) && 0 == sigaddset(&tmp_set, SIGPIPE)
+        && 0 == sigaddset(&tmp_set, SIGXFSZ)) {
+      (void)pthread_sigmask(SIG_UNBLOCK, &tmp_set, 0);
     }
   }
+  else {
+    if (0 == sigemptyset(&tmp_set) && 0 == sigpending(&tmp_set)) {
+      int chk;
+
+      chk = sigismember(&tmp_set, SIGPIPE);
+      if (0 == chk || (1 == chk && 0 == kill(pid, SIGPIPE))) {
+        chk = sigismember(&tmp_set, SIGXFSZ);
+        if ((0 == chk || (1 == chk && 0 == kill(pid, SIGXFSZ)))
+            && 0 == kill(pid, SIGUSR1)) {
+          pthread_exit(0);
+        }
+      }
+    }
+  }
+
   _exit(EX_FAIL);
 }
 
@@ -600,7 +617,7 @@ struct opts
       print_cctrs;      /* Print condition variable counters, -S. */
 };
 
-static const char version[] = "0.20";
+static const char version[] = "0.23";
 
 /* Backlog factor for all workers together. */
 static const unsigned blf = 4u;
@@ -635,7 +652,8 @@ usage(unsigned mx_worker)
 {
   log_info(
     "lbzip2: Parallel bzip2 utility.\n"
-    "Copyright (C) 2008, 2009 Laszlo Ersek. Released under the GNU GPLv2+.\n"
+    "Copyright (C) 2008, 2009, 2010 Laszlo Ersek.\n"
+    "Released under the GNU GPLv2+.\n"
     "Version %s.\n"
     "\n"
     "Usage:\n"
@@ -1523,7 +1541,12 @@ process(const struct opts *opts, unsigned num_slot, int infd, const char *isep,
     case CS_TERM:
       if (0 != opathn) {
         (void)unlink(opathn);
-        (*freef)(opathn);
+        /*
+          Don't release "opathn" -- the muxer might encounter a write error and
+          access *opathn via "ofmt" for error reporting before the main thread
+          re-raises the signal here. This is a deliberate leak, but we're on
+          our (short) way out anyway. 16-Feb-2010 lacos
+        */
         opathn = 0;
       }
 
@@ -1624,8 +1647,26 @@ main(int argc, char **argv)
     }
   }
 
-  xsigaction(SIGPIPE, SIG_IGN);
-  xsigaction(SIGXFSZ, SIG_IGN);
+  /*
+    SIGPIPE and SIGXFSZ will be blocked in all sub-threads during the entire
+    lifetime of the process. Any EPIPE or EFBIG write() condition will be
+    handled just as before lbzip2-0.23, when these signals were ignored.
+    However, starting with lbzip2-0.23, SIGPIPE and/or SIGXFSZ will be
+    generated for the offending thread(s) in addition. bailout(), when called
+    by such sub-threads in response to EPIPE or EFBIG, or when called by other
+    sub-threads failing concurrently (but a bit later) for any other reason,
+    will forward (regenerate) the pending signal(s) for the whole process. The
+    main thread will unblock these signals right before exiting with EX_FAIL in
+    bailout(). 2010-03-03 lacos
+  */
+  {
+    sigset_t base_mask;
+
+    xsigemptyset(&base_mask);
+    xsigaddset(&base_mask, SIGPIPE);
+    xsigaddset(&base_mask, SIGXFSZ);
+    xsigmask(SIG_BLOCK, &base_mask, 0);
+  }
 
   opts_setup(&opts, &operands, argc, argv);
 
